/***********************************************************
 * This file is part of glyr
 * + a command-line tool and library to download various sort of musicrelated metadata.
 * + Copyright (C) [2011-2012]  [Christopher Pahl]
 * + Hosted at: https://github.com/sahib/glyr
 *
 * glyr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glyr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glyr. If not, see <http://www.gnu.org/licenses/>.
 **************************************************************/

#ifndef APIKEYS_H
#define APIKEYS_H

/*
 * Please register your own keys if you fork parts of this code!
 * */
#define API_KEY_AMAZON  "AKIAJ6NEA642OU3FM24Q"
#define API_KEY_LASTFM  "7199021d9c8fbae507bf77d0a88533d7"
#define API_KEY_FLICKR  "b5af0c3230fb478d53b20835223d57a4"
#define API_KEY_GTRANS  "AIzaSyDzfBNb-W5G9pvZY6KGbEoRK4JvOIhUsjI"
#define API_KEY_HTBACK  "b3085ed18168f083aa69179b3364c9d8"
#define API_KEY_ECHONEST "ZSIUEIVVZGJVJVWIS"

#endif
